/*----------------------------------------------------------------------------
 *      RL-ARM - RTX
 *----------------------------------------------------------------------------
 *      Name:    Blinky.c
 *      Purpose: RTX example program
 *----------------------------------------------------------------------------
 *      This code is part of the RealView Run-Time Library.
 *      Copyright (c) 2004-2016 KEIL - An ARM Company. All rights reserved.
 *---------------------------------------------------------------------------*/

#include "cmsis_os.h"
#include "stm32l1xx.h"                  // Device header
#include "Board_LED.h"                  // ::Board Support:LED

osThreadId tid_ledOn;                   /* Thread id of thread: ledOn */
osThreadId tid_ledOff;                  /* Thread id of thread: ledOff */


/*----------------------------------------------------------------------------
  Thread 1 'ledOn': switches the LED on
 *---------------------------------------------------------------------------*/
void ledOn (void const *argument) {
  for (;;) {
    LED_On(0);                              /* Turn LED On */
    osSignalSet(tid_ledOff, 0x0001);        /* set signal to thread 'ledOff' */
    osDelay (500);                          /* delay 50 clock ticks */
  }
}

/*----------------------------------------------------------------------------
  Thread 2 'ledOff': switches the LED off
 *---------------------------------------------------------------------------*/
void ledOff (void const *argument) {
  for (;;) {
    osSignalWait(0x0001, osWaitForever);    /* wait for an event flag 0x0001 */
    osDelay (80);                           /* delay 8 clock ticks */
    LED_Off(0);                             /* Turn LED Off */
  }
}

/*----------------------------------------------------------------------------
  configure SystemCoreClock:
     System Clock source         PLL(HSI)
     SYSCLK                      32000000 Hz
     HCLK                        32000000 Hz
     AHB Prescaler               1
     APB1 Prescaler              1
     APB2 Prescaler              1
     HSI Frequency               16000000 Hz
     PLL DIV                     3
     PLL MUL                     6
     Flash Latency               1 WS
     SDIO clock (SDIOCLK)        48000000 Hz
     Require 48MHz for USB clock Disabled
 *----------------------------------------------------------------------------*/
static void SystemCoreClockConfigure(void) {

  RCC->CR |= ((uint32_t)RCC_CR_HSION);               /* Enable HSI */
  while ((RCC->CR & RCC_CR_HSIRDY) != RCC_CR_HSIRDY) __NOP();

  FLASH->ACR |= FLASH_ACR_ACC64;                     /* Enable 64-bit access */
  FLASH->ACR |= FLASH_ACR_PRFTEN;                    /* Enable Prefetch Buffer */
  FLASH->ACR |= FLASH_ACR_LATENCY;                   /* Flash 1 wait state */

  RCC->APB1ENR |= RCC_APB1ENR_PWREN;                 /* Enable the PWR APB1 Clock         */
  PWR->CR = PWR_CR_VOS_0;                            /* Select the Voltage Range 1 (1.8V) */
  while((PWR->CSR & PWR_CSR_VOSF) != 0);             /* Wait for Voltage Regulator Ready  */

  RCC->CFGR |= (RCC_CFGR_HPRE_DIV1  |                /* HCLK = SYSCLK / 1 */
                RCC_CFGR_PPRE2_DIV1 |                /* PCLK2 = HCLK / 1 */
                RCC_CFGR_PPRE1_DIV2  );              /* PCLK1 = HCLK / 2 */
 
  RCC->CFGR &= ~(RCC_CFGR_PLLSRC   |                 /* clear settings */
                 RCC_CFGR_PLLMUL   |
                 RCC_CFGR_PLLDIV    );
  RCC->CFGR |=  (RCC_CFGR_PLLSRC_HSI |               /* configure PLL */
                 RCC_CFGR_PLLDIV3    |
                 RCC_CFGR_PLLMUL6     );


  RCC->CR |= RCC_CR_PLLON;                           /* Enable PLL */
  while((RCC->CR & RCC_CR_PLLRDY) == 0) __NOP();     /* Wait till PLL is ready */

  RCC->CFGR &= ~(RCC_CFGR_SW);                       /* clear settings */
  RCC->CFGR |= RCC_CFGR_SW_PLL;                      /* Select PLL as system clock source */
  while ((RCC->CFGR & (uint32_t)RCC_CFGR_SWS) != RCC_CFGR_SWS_PLL) __NOP();
}



osThreadDef(ledOn, osPriorityNormal, 1, 0);
osThreadDef(ledOff, osPriorityNormal, 1, 0);

/*----------------------------------------------------------------------------
  Main: Initialize and start RTX Kernel
 *---------------------------------------------------------------------------*/
int main (void) {

  SystemCoreClockConfigure();

  LED_Initialize();                                  /* Initialize LEDs */
  
  tid_ledOn  = osThreadCreate(osThread(ledOn), NULL);
  tid_ledOff = osThreadCreate(osThread(ledOff), NULL);

  osDelay(osWaitForever);
  while(1);
}
